// Copyright 2019 Radulescu Bogdan Alexandru, 331CA

#include <stdio.h>
#include <stdlib.h>
#include <avr/io.h>
#include <time.h>
#include <avr/interrupt.h>
#include <avr/pgmspace.h>
#include <util/delay.h>
#include <string.h>
#include "nokia5110/nokia5110.h"
#include "adxl345/adxl345.h"

#define _L 0
#define _C 1
#define _R 2

#define _freq 70

int button[3];
int modulo = 6;
double roll_time = 3.0f;
int data;
enum state_list {
    INIT,
    MENU,
    ROLL,
    FINAL,
    CHOOSE_TIME,
    CHOOSE_DIE
};

enum state_list state = INIT;

int is_pressed(int b) {
    int p = b == _L ? PB0 :
            b == _C ? PB1 : 
            PB3;
    return !(PINB & (1 << p));
}

void clear_btn() {
    for (int i = 0; i < 3; i++) {
        if (!is_pressed(i))
            button[i] = 0;
    }
}

void switch_state() {
    if (state == INIT) {
        if (is_pressed(_C) && !button[_C]) {
            button[_C] = 1;
            state = MENU;
        }
    }
    if (state == MENU) {
        if (is_pressed(_C) && !button[_C]) {
            button[_C] = 1;
            state = ROLL;
        }
        if (is_pressed(_L) && !button[_L]) {
            button[_L] = 1;
            state = CHOOSE_TIME;
        }
        if (is_pressed(_R) && !button[_R]) {
            button[_R] = 1;
            state = CHOOSE_DIE;
        }
    }
    if (state == FINAL) {
        if (is_pressed(_C) && !button[_C]) {
            button[_C] = 1;
            state = ROLL;
        }
        if (is_pressed(_L) && !button[_L]) {
            button[_L] = 1;
            state = MENU;
        }
    }
    if (state == CHOOSE_DIE) {
        if (is_pressed(_C) && !button[_C]) {
            button[_C] = 1;
            state = MENU;
        }
    }
    if (state == CHOOSE_TIME) {
        if (is_pressed(_C) && !button[_C]) {
            button[_C] = 1;
            state = MENU;
        }
    }
}

void execute_init() {
    if (state != INIT)
        return;
    nokia_lcd_clear();
    nokia_lcd_set_cursor(0, 0);
    nokia_lcd_write_string("Radulescu Bogdan 331CA", 1);
    nokia_lcd_set_cursor(0, 16);
    nokia_lcd_write_string("Electronic Die", 1);
    nokia_lcd_set_cursor(0, 32);
    nokia_lcd_write_string("(C) to continue", 1);
    nokia_lcd_render();
}

double hash(int x, int y, int z) {
    return (double) 7 * x + 9 * y + 11 * z;
}

void execute_roll() {
    if (state != ROLL)
        return;
    double res = 0.0f;
    double ax, ay, az;
    char result[31];
    for (unsigned long i = 0; i < (unsigned long) _freq * roll_time; i++) {
        nokia_lcd_clear();
        nokia_lcd_set_cursor(30, 20);
        memset(&result, 0, 31);
        adxl345_getdata(&ax, &ay, &az);
        res += hash((int) ax, (int) ay, (int) az);
        data = (unsigned int) res % modulo + 1;
        sprintf(result, "%d", data);
        nokia_lcd_write_string(result, 2);
        nokia_lcd_render();
    }
    state = FINAL;
}

void execute_final() {
    if (state != FINAL)
        return;
    nokia_lcd_clear();
    char result[31];
    memset(&result, 0, 31);
    sprintf(result, "%d", data);
    nokia_lcd_set_cursor(0, 0);
    if (data == 1) {
        nokia_lcd_write_string("* CRIT. MISS *", 1);
    }
    if (data == modulo) {
        nokia_lcd_write_string("* CRIT.  HIT *", 1);
    }
    nokia_lcd_set_cursor(30, 15);
    nokia_lcd_write_string(result, 2);
    nokia_lcd_set_cursor(0, 30);
    nokia_lcd_write_string("(L) to menu", 1);
    nokia_lcd_set_cursor(0, 38);
    nokia_lcd_write_string("(C) to roll", 1);
    nokia_lcd_render();
}

void execute_menu() {
    if (state != MENU)
        return;
    nokia_lcd_clear();
    nokia_lcd_set_cursor(0, 0);
    nokia_lcd_write_string("(L) to change time (3s)", 1);
    nokia_lcd_set_cursor(0, 16);
    nokia_lcd_write_string("(C) to roll", 1);
    nokia_lcd_set_cursor(0, 32);
    nokia_lcd_write_string("(R) to change # of faces (6)", 1);
    nokia_lcd_render();
}

void execute_choose_time() {
    if (state != CHOOSE_TIME)
        return;
    if (is_pressed(_L) && !button[_L]) {
        button[_L] = 1;
        roll_time = roll_time >= 1.1f ? roll_time - 0.1f : 1.0f;
    }
    if (is_pressed(_R) && !button[_R]) {
        button[_R] = 1;
        roll_time = roll_time <= 9.8 ? roll_time + 0.1f : 9.9f;
    }
    nokia_lcd_clear();
    nokia_lcd_set_cursor(10, 14);
    char text[30];
    memset(text, 0, 30);
    sprintf(text, "Time: ");
    int integer = (int) roll_time + 48;
    int point = 46;
    int decimal = (int)(roll_time * 10) % 10 + 48;
    nokia_lcd_write_string(text, 1);
    nokia_lcd_write_char(integer, 1);
    nokia_lcd_write_char(point, 1);
    nokia_lcd_write_char(decimal, 1);
    nokia_lcd_write_string(" s", 1);
    nokia_lcd_set_cursor(0, 40);
    nokia_lcd_write_string("(C) to confirm", 1);
    nokia_lcd_render();
}
void execute_choose_die() {
    if (state != CHOOSE_DIE)
        return;
    if (is_pressed(_L) && !button[_L]) {
        button[_L] = 1;
        modulo = modulo > 2 ? modulo - 1 : 2;
    }
    if (is_pressed(_R) && !button[_R]) {
        button[_R] = 1;
        modulo++;
    }
    nokia_lcd_clear();
    nokia_lcd_set_cursor(20, 14);
    char text[30];
    memset(text, 0, 30);
    sprintf(text, "Die: d%d", modulo);
    nokia_lcd_write_string(text, 1);
    nokia_lcd_set_cursor(0, 40);
    nokia_lcd_write_string("(C) to confirm", 1);
    nokia_lcd_render();
}

void interpret_state() {
    switch(state) {
        case MENU: execute_menu();
        case ROLL: execute_roll();
        case FINAL: execute_final();
        case INIT: execute_init();
        case CHOOSE_DIE: execute_choose_die();
        case CHOOSE_TIME: execute_choose_time();
    }
}

int main(void) 
{

    nokia_lcd_init();  // init lcd
    nokia_lcd_clear();
    adxl345_init();  // init gyro / accelerom module

    // Init buttons

    DDRB  &= ~(1 << PB0);
    PORTB |= (1 << PB0);
    DDRB  &= ~(1 << PB1);
    PORTB |= (1 << PB1);
    DDRB  &= ~(1 << PB3);
    PORTB |= (1 << PB3);

    while(1) {
        clear_btn();
        interpret_state();
        switch_state();
    }

}
